/*
 *************************************************************************
 *
 * serdisp_connect.h
 * routines for accessing supported devices (parport, serial device, ..)
 *
 *************************************************************************
 *
 * copyright (C) 2003-2005  wolfgang astleitner
 * email     mrwastl@users.sourceforge.net
 *
 *************************************************************************
 *
 * parport part initially based on:
 *   http://www.thiemo.net/projects/orpheus/optrex/
 *
 *************************************************************************
 * This program is free software; you can redistribute it and/or modify   
 * it under the terms of the GNU General Public License as published by   
 * the Free Software Foundation; either version 2 of the License, or (at  
 * your option) any later version.                                        
 *                                                                        
 * This program is distributed in the hope that it will be useful, but    
 * WITHOUT ANY WARRANTY; without even the implied warranty of             
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      
 * General Public License for more details.                               
 *                                                                        
 * You should have received a copy of the GNU General Public License      
 * along with this program; if not, write to the Free Software            
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA              
 * 02111-1307, USA.  Or, point your browser to                            
 * http://www.gnu.org/copyleft/gpl.html                                   
 *************************************************************************
 */


#ifndef SERDISP_CONNECT_H
#define SERDISP_CONNECT_H


#include <termios.h>
#include <signal.h>


/* define 'byte' if not available yet */
#ifndef byte
  #define byte unsigned char
#endif


/* connection types (for bit-field) */
#define SERDISPCONNTYPE_PARPORT           1
#define SERDISPCONNTYPE_SERRAW            2
#define SERDISPCONNTYPE_I2C               4

/* short forms for connection types */
#define SDCT_PP                           SERDISPCONNTYPE_PARPORT
#define SDCT_SRAW                         SERDISPCONNTYPE_SERRAW
#define SDCT_I2C                          SERDISPCONNTYPE_I2C


/* *************************** *
 * constants for parallel port *
 * *************************** */

/* base+0 ... data bits (in/out) */
#define SD_PP_D0    0x00000001
#define SD_PP_D1    0x00000002
#define SD_PP_D2    0x00000004
#define SD_PP_D3    0x00000008
#define SD_PP_D4    0x00000010
#define SD_PP_D5    0x00000020
#define SD_PP_D6    0x00000040
#define SD_PP_D7    0x00000080

/* base+1 ... status bits (in) */
#define SD_PP_S0    0x00000100
#define SD_PP_S1    0x00000200
#define SD_PP_S2    0x00000400
#define SD_PP_S3    0x00000800  /* pin 15: ERROR */
#define SD_PP_S4    0x00001000  /* pin 13: SLCT */
#define SD_PP_S5    0x00002000  /* pin 12: PE */
#define SD_PP_S6    0x00004000  /* pin 10: ACK */
#define SD_PP_S7    0x00008000  /* pin 11: /BUSY */

/* base+2 ... control bits (out) */
#define SD_PP_C0    0x00010000  /* pin  1: /STROBE */
#define SD_PP_C1    0x00020000  /* pin 14: /LINEFD */
#define SD_PP_C2    0x00040000  /* pin 16: INIT */
#define SD_PP_C3    0x00080000  /* pin 17: /SELECT */
#define SD_PP_C4    0x00100000
#define SD_PP_C5    0x00200000  /* enable bi-directional port */
#define SD_PP_C6    0x00400000
#define SD_PP_C7    0x00800000

#define SD_PP_BIDI  0x20        /* bi-di bit 5 ( ==  SD_PP_C5 >> 16 ) */
/* active low signals on a centronics parallel port */
#define SD_PP_ACTIVELOW  0x000B8000

/* input/output flags  (io_flags) */
#define SD_PP_WRITEDB        1  /* parport base+0 (data byte) needs to be written */
#define SD_PP_WRITECB        2  /* parport base+2 (control byte) -"-              */
#define SD_PP_READDB         4  /* parport base+0 (data byte) needs to be read */
#define SD_PP_READSB         8  /* parport base+1 (status byte) -"- */
#define SD_PP_READCB        16  /* parport base+2 (control byte) -"- */

#define SD_SP_TXDON          1  /* serport: keep TxD high for gaining power */

/* no constants for serial port (used one byte only) */


#define SD_MAX_SUPP_SIGNALS 32   /* maximum supported signals */


/* struct containing all infos and other things concerning a display */
typedef struct serdisp_CONN_s {
  int conntype;                       /* one of SERDISPCONNTYPE_* */

  long signals[SD_MAX_SUPP_SIGNALS];  /* lookup vector for signals */
  long signals_permon;                /* signals forced to be permanently high */
  long signals_invert;                /* signals to be inverted (because they're inverted by some hardware or somethink like that
                                         (default: 0L -> no inversions) */


                                      /* 8 flags freely accessible for different SERDISCONNTYPEs */
  byte io_flags_readstatus;
  byte io_flags_writedata;
  byte io_flags_writecmd;

  byte io_flags_default;              /* use this as default if io_flags_readstatus/writedata/writecmd not set */


  /* parport and serport variables */
  int  pp_bidi_supported;             /* parport: 1: bidirectional access supported, 0: not supported */
  byte pp_ctrlbits_saved;             /* parport: control bits saved (state after initialising the parport) */

  int directIO;                       /* 1: direct IO, 0: ioctl-calls */
  unsigned short int port;            /* port if access using direct io */
  int fd;                             /* file-descriptor if access using ioctl-calls */

  /* serport only */
  struct termios termstate_bkp;       /* backuped state of serport */
  struct termios termstate;           /* state of serport */

  char*          sdcdev;              /* unprocessed device name (including connection type) */
  
  int            debug_count;         /* counter for debugging purposes */
} serdisp_CONN_t;



typedef struct serdisp_conntype_s {
  char* connname;
  int conntype;
  char* description;
} serdisp_conntype_t;



typedef struct serdisp_wiresignal_s {
  int conntype;
  char* signalname;
  int   activelow;
  char  cord;           /* command 'C' or data 'D' signal */
  int   index;
} serdisp_wiresignal_t;



typedef struct serdisp_wiredef_s {
  int id;
  int conntype;
  char* name;
  char* definition;
  char* description;
} serdisp_wiredef_t;



#ifdef __cplusplus
extern "C" {
#endif
  serdisp_CONN_t* SDCONN_open      (const char sdcdev[]);
  void            SDCONN_close     (serdisp_CONN_t* sdcd);
  void            SDCONN_write     (serdisp_CONN_t* sdcd, long data, byte flags);
  long            SDCONN_read      (serdisp_CONN_t* sdcd, byte flags);
  serdisp_CONN_t* SDCONN_import_PP (int directIO, int hport);
  
  int             SDCONN_getsignalindex    (const char str[], int signaltype);
  long            SDCONN_getsignalvalue    (int idx);
  int             SDCONN_issignalacticelow (int idx);
  char*           SDCONN_getsignalname     (int idx);
  int             SDCONN_isactivelow       (long signal, int signaltype);
  
  
#ifdef __cplusplus
    }
#endif




#endif /* SERDISP_CONNECT_H */
